body {
    background-color: #050c22;
    margin: 5%;
  }

.flex-display {
  display: flex;
}

#header {
align-items: center;
}

#mainContainer {
    display: flex;
    flex-direction: column;
}

#title {
    text-align: center;
  }
  
#title1 {
  text-align: center;
}

.custom-tabs-container {
  width: 100%;
}

.custom-tabs {
  color:#050c22;
  background-color: #050c22;
  padding: 0px 24px;
  width: 100%;
}

.custom-tab {
  color:#b4b5c2;
  font-family: 'Work Sans';
  font-size: 22px;
  background-color: #0f0471;
  border-top-left-radius: 3px;
  border-top-right-radius: 3px;
  border-top: 3px solid transparent !important;
  border-left: 0px !important;
  border-right: 0px !important;
  border-bottom: 0px !important;
  background-color: #050c22;
  padding: 12px !important;
  font-family: "system-ui";
  display: flex !important;
  align-items: center;
  justify-content: center;
}

.card_container {
  border-radius: 5px;
  background-color: #119DFF;
  margin: 16px;
  padding: 15px;
  position: relative;
  box-shadow: 2px 2px 2px #119DFF;
}

.fix_label {
  font-size: 20px;
    margin: 0;
    padding: 10px;
    padding-bottom: 0px;
    margin-bottom: 0px;
    width: calc(100%-40px);
  }

.dcc_compon {
  margin: 0;
  padding: 5px;
  width: calc(100%-40px);
}

.dropdown {
  /* border-radius: 5px; */
  background-color: #1F75FE;
  margin: 15px;
  padding: 15px;
  /* position: relative; */
  /* box-shadow: 2px 2px 2px #07380d; */
}

.custom-tab--selected {
  /* border-left: 1px solid #dc09e3 !important; */
  /* border-right: 1px solid #dc09e3 !important; */
  /* border-top: 3px solid #dc09e3 !important; */
}


/* .tabs_styles {
  height: 44px;
  align-items: center;
}

.tab_style {
  borderBottom: 1px solid #d6d6d6;
  padding: 6px;
  fontWeight: bold;
  border-radius: 15px;
  background-color: #F2F2F2;
  box-shadow: 4px 4px 4px 4px lightgrey;
}

.tab_selected_style {
  borderTop: 1px solid #d6d6d6;
  borderBottom: 1px solid #d6d6d6;
  backgroundColor: #119DFF;
  color: white;
  padding: 6px;
  border-radius: 15px;
} */